from django.contrib import admin

from feincms_oembed.models import CachedLookup


class CachedLookupAdmin(admin.ModelAdmin):
    list_display = (
        'url', 'created', 'modified', 'max_age_seconds', '_httpstatus')
    fields = ('hash', 'url', 'max_age_seconds', '_httpstatus', '_response')
    readonly_fields = ('_response', '_httpstatus')


admin.site.register(CachedLookup, CachedLookupAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CachedLookup',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('hash', models.CharField(help_text='SHA-1 hash of the URL.', unique=True, max_length=40, verbose_name='hash')),
                ('url', models.URLField(max_length=1000, verbose_name='URL')),
                ('_response', models.TextField(null=True, blank=True)),
                ('_httpstatus', models.PositiveIntegerField(null=True, blank=True)),
                ('max_age_seconds', models.PositiveIntegerField(default=604800, verbose_name='Max. age in seconds')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', models.DateTimeField(auto_now=True, verbose_name='modified')),
            ],
            options={
                'verbose_name': 'cached lookup',
                'verbose_name_plural': 'cached lookups',
            },
            bases=(models.Model,),
        ),
    ]



from django.utils.http import urlencode
from django.conf import settings


def embedly_oembed_provider(url, kwargs):
    """
    Provider for the oEmbed service at http://embed.ly/
    Embedly requires an API key.
    """
    kwargs['url'] = url
    kwargs['key'] = settings.EMBEDLY_KEY
    return 'https://api.embed.ly/1/oembed?%s' % urlencode(kwargs)


def noembed_oembed_provider(url, kwargs):
    """
    Provider for the oEmbed service at http://noembed.com/
    """
    kwargs['url'] = url
    return 'https://noembed.com/embed?%s' % urlencode(kwargs)



import hashlib
try:
    from importlib import import_module
except ImportError:  # PY2.6
    from django.utils.importlib import import_module

import json
try:
    from urllib.request import URLError, urlopen
except ImportError:  # PY3
    from urllib2 import URLError, urlopen

from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.utils import six, timezone
from django.utils.translation import ugettext_lazy as _


DEFAULT_MAX_AGE = 7 * 24 * 60 * 60  # Cache lookups for a week


class CachedLookupManager(models.Manager):
    _oembed_provider_fn = None

    def oembed_provider(self, url, kwargs):
        """
        Helper method returning the oEmbed provider function
        """

        if not self._oembed_provider_fn:
            self._oembed_provider_fn = _get_object(getattr(
                settings,
                'OEMBED_PROVIDER',
                'feincms_oembed.providers.embedly_oembed_provider',
            ))
        return self._oembed_provider_fn(url, kwargs)

    def get_by_url(self, url, max_age=DEFAULT_MAX_AGE):
        lookup, created = self.get_or_create(
            hash=hashlib.sha1(url.encode('utf-8')).hexdigest(),
            max_age_seconds=max_age,
            defaults={
                'url': url,
            },
        )

        if created:
            lookup.clean()
            lookup.save()

        return lookup

    def request(self, url, max_age=DEFAULT_MAX_AGE):
        return self.get_by_url(url, max_age).response

    def oembed(self, url, max_age=DEFAULT_MAX_AGE, **kwargs):
        lookup = self.get_by_url(
            self.oembed_provider(url, kwargs),
            max_age=max_age,
        )

        response = json.loads(lookup.response)
        try:
            response['modified'] = lookup.modified
        except AttributeError:
            pass
        return response


class CachedLookup(models.Model):
    hash = models.CharField(
        _('hash'), max_length=40, unique=True,
        help_text=_('SHA-1 hash of the URL.'))
    url = models.URLField(_('URL'), max_length=1000)
    _response = models.TextField(blank=True, null=True)
    _httpstatus = models.PositiveIntegerField(blank=True, null=True)

    max_age_seconds = models.PositiveIntegerField(
        _('Max. age in seconds'), default=DEFAULT_MAX_AGE)

    created = models.DateTimeField(_('created'), auto_now_add=True)
    modified = models.DateTimeField(_('modified'), auto_now=True)

    class Meta:
        verbose_name = _('cached lookup')
        verbose_name_plural = _('cached lookups')

    objects = CachedLookupManager()

    @property
    def response(self):
        delta = timezone.now() - self.modified

        if delta.seconds > self.max_age_seconds:
            self.clean()
            self.save()

        return self._response

    def clean(self, *args, **kwargs):
        try:
            request = urlopen(self.url)
        except URLError as e:
            raise ValidationError(
                u'This URL cannot be requested: %s' % self.url, e)

        raw = request.read()

        try:
            decoded = raw.decode('utf-8')
        except UnicodeDecodeError:
            decoded = raw.decode('iso8859-1')
        self._response = decoded
        self._httpstatus = request.getcode()

    def __unicode__(self):
        return self.url


def _get_object(path, fail_silently=False):
    # Return early if path isn't a string (might already be an callable or
    # a class or whatever)
    if not isinstance(path, six.string_types):  # XXX bytes?
        return path

    try:
        return import_module(path)
    except ImportError:
        try:
            dot = path.rindex('.')
            mod, fn = path[:dot], path[dot + 1:]

            return getattr(import_module(mod), fn)
        except (AttributeError, ImportError):
            if not fail_silently:
                raise



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'CachedLookup'
        db.create_table('feincms_oembed_cachedlookup', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('hash', self.gf('django.db.models.fields.CharField')(unique=True, max_length=40)),
            ('url', self.gf('django.db.models.fields.URLField')(max_length=1000)),
            ('_response', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('_httpstatus', self.gf('django.db.models.fields.PositiveIntegerField')(null=True, blank=True)),
            ('max_age_seconds', self.gf('django.db.models.fields.PositiveIntegerField')(default=604800)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('modified', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
        ))
        db.send_create_signal('feincms_oembed', ['CachedLookup'])

    def backwards(self, orm):
        # Deleting model 'CachedLookup'
        db.delete_table('feincms_oembed_cachedlookup')

    models = {
        'feincms_oembed.cachedlookup': {
            'Meta': {'object_name': 'CachedLookup'},
            '_httpstatus': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            '_response': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'hash': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '40'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'max_age_seconds': ('django.db.models.fields.PositiveIntegerField', [], {'default': '604800'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '1000'})
        }
    }

    complete_apps = ['feincms_oembed']


from django.core.exceptions import ValidationError
from django.db import models
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _

from feincms_oembed.models import CachedLookup


class OembedContent(models.Model):
    """
    Content type for integrating anything supported by the oembed provider into
    the CMS

    Usage::

        Page.create_content_type(OembedContent, TYPE_CHOICES=[
            ('default', _('Default presentation'), {
                'maxwidth': 500, 'maxheight': 300}),
            ('transparent', _('Transparent'), {
                'maxwidth': 500, 'maxheight': 300, 'wmode': 'transparent'}),
            ])
    """

    url = models.URLField(
        _('URL'),
        help_text=_(
            'Insert an URL to an external content you want to embed,'
            ' e.g. http://www.youtube.com/watch?v=Nd-vBFJN_2E'),
    )

    class Meta:
        abstract = True
        verbose_name = _('external content')
        verbose_name_plural = _('external contents')

    @classmethod
    def initialize_type(cls, TYPE_CHOICES, PARAMS={}):
        choices = [row[0:2] for row in TYPE_CHOICES]
        cls.add_to_class(
            'type',
            models.CharField(
                _('type'),
                max_length=20,
                choices=choices,
                default=choices[0][0],
            )
        )
        cls._type_config = dict((row[0], row[2]) for row in TYPE_CHOICES)
        cls._params = PARAMS

    def get_html_from_json(self, fail_silently=False):
        params = self._type_config.get(self.type, {})
        params.update(self._params)

        try:
            embed = CachedLookup.objects.oembed(self.url, **params)
        except TypeError:
            if fail_silently:
                return u''
            raise ValidationError(
                _('I don\'t know how to embed %s.') % self.url)

        return render_to_string((
            'content/external/%s.html' % embed.get('type'),
            'content/external/%s.html' % self.type,
            'content/external/default.html',
        ), {'response': embed, 'content': self})

    def clean(self, *args, **kwargs):
        self.get_html_from_json()

    def render(self, **kwargs):
        return self.get_html_from_json(fail_silently=True)


class FeedContent(models.Model):
    url = models.URLField(
        _('Feed URL'),
        help_text=_(
            'Insert any RSS URL,'
            ' e.g. https://www.djangoproject.com/rss/weblog/'),
    )

    class Meta:
        abstract = True
        verbose_name = _('RSS Feed')
        verbose_name_plural = _('RSS Feeds')

    def clean(self, *args, **kwargs):
        import feedparser
        feedparser.parse(CachedLookup.objects.request(self.url, 30 * 60))

    @property
    def feed(self):
        import feedparser
        return feedparser.parse(
            CachedLookup.objects.request(self.url, 30 * 60))

    def render(self, **kwargs):
        return render_to_string('content/external/feed.html', {
            'feed': self.feed,
        })


class OembedMixin(models.Model):
    """
    Mixin for usage in custom feincms content types
    Usage::

        Page.create_content_type(
            YourContentWithOembedMixin,
            OEMBED_PARAMS={
                'maxwidth': 500,
                'maxheight': 300,
                'wmode': 'transparent',
            },
        )

    Oembed content will be available via self.oembed
    """

    url = models.URLField(
        _('Video URL'),
        help_text=_(
            'Insert an URL to an external content you want to embed,'
            ' e.g. http://www.youtube.com/watch?v=Nd-vBFJN_2E'),
        blank=True,
    )

    class Meta:
        abstract = True

    @classmethod
    def initialize_type(cls, OEMBED_PARAMS={}):
        cls._params = OEMBED_PARAMS

    def get_html_from_json(self, fail_silently=False):
        if not self.url:
            self.oembed = ''
            return True

        params = {}
        params.update(self._params)

        try:
            embed = CachedLookup.objects.oembed(self.url, **params)
        except TypeError:
            if fail_silently:
                return u''
            raise ValidationError(
                _('I don\'t know how to embed %s.') % self.url)

        self.oembed = embed

    def process(self, request, **kwargs):
        self.get_html_from_json()
